CREATE TABLE IF NOT EXISTS `ShortRoleNames`
(
    `Id`             BIGINT                   NOT NULL AUTO_INCREMENT,
    `ShortName`      VARCHAR(255)             NOT NULL,
    `DiscordRoleId`  BIGINT                   NOT NULL,
    `Position`       ENUM ('before', 'after') NOT NULL,
    `ServerId`       BIGINT,
    `CreatedAt`      DATETIME(6)              NULL DEFAULT CURRENT_TIMESTAMP(6),
    `LastModifiedAt` DATETIME(6)              NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
    PRIMARY KEY (`Id`),
    FOREIGN KEY (`ServerId`) REFERENCES `Servers` (`ServerId`)
);


CREATE TABLE IF NOT EXISTS `ShortRoleNamesHistory`
(
    `Id`            BIGINT(20)               NOT NULL,
    `ShortName`     VARCHAR(64) DEFAULT NULL,
    `DiscordRoleId` BIGINT(20)               NOT NULL,
    `Position`      ENUM ('Before', 'After') NOT NULL,
    `ServerId`      BIGINT(20)  DEFAULT NULL,
    `Deleted`       BOOL        DEFAULT FALSE,
    `DateFrom`      DATETIME(6)              NOT NULL,
    `DateTo`        DATETIME(6)              NOT NULL
);;

DROP TRIGGER IF EXISTS `TR_ShortRoleNamesUpdate`;;

CREATE TRIGGER `TR_ShortRoleNamesUpdate`
    AFTER UPDATE
    ON `ShortRoleNames`
    FOR EACH ROW
BEGIN
    INSERT INTO `ShortRoleNamesHistory` (`Id`, `ShortName`, `DiscordRoleId`, `Position`, `ServerId`, `DateFrom`,
                                         `DateTo`)
    VALUES (OLD.Id, OLD.ShortName, OLD.DiscordRoleId, OLD.Position, OLD.ServerId, OLD.LastModifiedAt,
            CURRENT_TIMESTAMP(6));
END;;

DROP TRIGGER IF EXISTS `TR_ShortRoleNamesDelete`;;

CREATE TRIGGER `TR_ShortRoleNamesDelete`
    AFTER DELETE
    ON `ShortRoleNames`
    FOR EACH ROW
BEGIN
    INSERT INTO `ShortRoleNamesHistory` (`Id`, `ShortName`, `DiscordRoleId`, `Position`, `ServerId`, `Deleted`,
                                         `DateFrom`,
                                         `DateTo`)
    VALUES (OLD.Id, OLD.ShortName, OLD.DiscordRoleId, OLD.Position, OLD.ServerId, TRUE, OLD.LastModifiedAt,
            CURRENT_TIMESTAMP(6));
END;;

